#ifndef DATALANG_H_SEEN
#define DATALANG_H_SEEN

#include <glib.h>

typedef enum {
    DL_T_NULL,
    DL_T_BOOLEAN,
    DL_T_NUMBER,
    DL_T_BYTEARRAY,
    DL_T_STRING,
    DL_T_LIST,
    DL_T_RECORD
} DL_Type;

typedef struct DL_Value_ DL_Value;

typedef int     DL_Boolean;
typedef double  DL_Number;      // XXX use mpq_t (libgmp)

typedef struct {
    size_t len;
    const uint8_t *bytes;
} DL_ByteArray;

typedef struct {
    size_t len;
    const uint8_t *bytes;       // also null-terminated after len bytes
    const char *encoding;       // null-terminated; defaults to "utf8"
} DL_String;

typedef struct {
    size_t len;
    DL_Value **elems;
} DL_List;

typedef struct {
    GTree *tree;                // maps DL_String to DL_Value
} DL_Record;

struct DL_Value_ {
    DL_Type type;
    union {
        DL_Boolean      boolean;
        DL_Number       number;
        DL_ByteArray    bytearray;
        DL_String       string;
        DL_List         list;
        DL_Record       record;
    };
};

#endif
